-- Retrieve the imported UI Package Field
local UIField = script.fields.UIField
-- Access the YaResourceManager API
local YaResourceManager = YahahaMiddleLayerSlim.Resource.YaResourceManager 
local mainPanel
local packageName

-- Load resources when the game starts
script.OnStart(function ()
    LoadResource()
end)

-- Function to load resources
function LoadResource()
    -- Use the LoadResourceByUIPackageField interface to load the specified UIField.
    -- The function returns the loading state at each stage:
    -- - AssetStatus.ManifestBytesCompleted: Serialized bytes or manifest loaded.
    -- - AssetStatus.BaseAssetCompleted: Bytes files loaded.
    -- - AssetStatus.AllAssetCompleted: All files loaded.
    -- packageName is the name of the current package (it is only returned during BaseAssetCompleted and AllAssetCompleted stages; it is null during ManifestBytesCompleted stage).
    YaResourceManager.LoadResourceByUIPackageField(UIField, function(state, name)
        -- When all resources are loaded, create the specified Panel (i.e., when state == AssetStatus.AllAssetCompleted).
        if state == AssetStatus.AllAssetCompleted then
            packageName = name

            -- Create the main panel
            CreateMainPanel()
        end
    end)
end

function CreateMainPanel()
    -- Create the MainPanel from the Sample Package
    mainPanel = UIPackage.CreateObject(packageName, "MainPanel")
    -- Set the screen resolution scale based on your panel's resolution.
    -- GRoot.inst is the built-in GUI rendering root node.
    GRoot.inst:SetContentScaleFactor(1920, 1080)
    -- Add the UI panel to the GUI root node to display it on screen.
    GRoot.inst:AddChild(mainPanel)

    -- Set the panel size to match the screen size.
    mainPanel.size = GRoot.inst.size
    -- Establish a two-way binding relationship between the panel and the screen size; when the screen size changes, so does the panel.
    mainPanel:AddRelation(GRoot.inst, RelationType.Size)

    -- Below are samples for each feature:
    -- Transition
    ShowStartGame()
    -- Image
    ChangeMapImage()
    -- List
    ShowRankList()
    -- Button and Controller
    Shooting()
    -- InputField and RichText
    ChatInput()
    -- DropDown and Loader
    DropDownAndLoader()
    -- Graph
    ChangeGraphColor()
    -- Create Sub Panel
    AddGoldOrGem()
end

-- System's Update method
script.OnUpdate(function ()
    -- Textfield and Progress
    CountDownText()

    -- Shooting Countdown
    ShootingCountDown()
end)

-- Transition: Play opening animation
function ShowStartGame()
    -- Use GetChild method to get "StartComp" component.
    local startComp = mainPanel:GetChild("StartGame")
    -- Get "start" transition from "StartComp" component.
    local startTransition = startComp:GetTransition("start")
    
    -- Method to hide StartComp.
    local function HideStartComp()
        startComp.visible = false
    end
    
    -- Play startTransition and execute HideStartComp when animation ends.
    startTransition:Play(HideStartComp)
end

-- Image: Modify image fill ratio
function ChangeMapImage()
    -- Use GetChildByPath method to get "FillImage" from "ImageAndSlider" component.
    local image = mainPanel:GetChildByPath("ImageAndSlider.FillImage")
    
    -- Get "FillValueSlider" from "ImageAndSlider" component.
    local slider = mainPanel:GetChildByPath("ImageAndSlider.FillValueSlider")
    
    -- Get "FillImageText" from "ImageAndSlider" component.
    local text = mainPanel:GetChildByPath("ImageAndSlider.FillImageText")
    
    local function OnSliderMoveEvent()
        -- Modify fill ratio value.
        image.fillAmount = slider.value / 100           
        -- Update fill ratio text.
        text.text = math.ceil(slider.value).."%"         
    end
    
   slider.onChanged:AddListener0(OnSliderMoveEvent)  -- Listen for slider drag events; update image fill value and text when dragged.
end

-- List: Create a player ranking list
function ShowRankList()
   -- Create player list data.
   local rankData = {} 
   -- Get list by path.
   local list = mainPanel:GetChildByPath("RankListComp.RankList")           
   -- Get Add Button.
   local addButton = mainPanel:GetChildByPath("RankListComp.AddPlayerButton")      
   -- Get Remove Button.
   local removeButton = mainPanel:GetChildByPath("RankListComp.RemovePlayerButton")     
   -- Refresh list data; set item count to 0.
   list.numItems = 0       

   -- Method to add player information.
   local function AddButtonOnClickEvent() 
       local length = rankData and #rankData or 1
       local newPlayer = {
           name = "hahaha_"..length,
           score = math.random(1, 100)
       }
       table.insert(rankData, newPlayer)
       table.sort(rankData, function (a, b)
           return a.score > b.score  -- Sort players by score in descending order.
       end)

       -- After updating player data, modify list's numItems to refresh it.
       list.numItems = #rankData
   end
   
   addButton.onClick:AddListener0(AddButtonOnClickEvent)  -- Bind Add button click event.      

   -- Method to remove player information.
   local function RemoveButtonOnClickEvent()
       if rankData and #rankData > 0 then
           table.remove(rankData, 1)  -- Remove first player in rankData.
       end
      
       list.numItems = #rankData  -- Refresh list after updating player data.
   end
   
   removeButton.onClick:AddListener0(RemoveButtonOnClickEvent)  -- Bind Remove button click event. 

   -- OnRenderList will return each Item and its corresponding Index for user modification.
   local function OnRenderList(index, component)
       component:GetChild("Rank").text = tostring(index)  -- Set rank number based on index.
       component:GetChild("Name").text = rankData[index + 1].name  -- Set player name from rankData.
       component:GetChild("Num").text = tostring(rankData[index + 1].score)  -- Set player score from rankData.
   end
   
   list.itemRenderer = OnRenderList  -- Bind item rendering method to list.                      
end

-- Button and Controller functionality
local isShooting = true  -- Flag for shooting status.
local text  -- Variable for displaying remaining bullets text.
local shootingCountDown  -- Variable for countdown timer text field.
local BulletCount  -- Variable for bullet count tracking.
local controller1  -- Controller for shooting image state.
local controller2  -- Controller for attack button state.

function Shooting()
   local button = mainPanel:GetChildByPath("ButtonComp.AttackButton")  -- Get button by path.

   controller1 = mainPanel:GetChildByPath("ButtonComp.ShootingImage"):GetController("isShooting")  -- Get shooting image controller by path.

   controller2 = mainPanel:GetChildByPath("ButtonComp.AttackButton"):GetController("isShooting")  -- Get attack button controller by path.

   shootingCountDown = mainPanel:GetChildByPath("ButtonComp.AttackButton.ShootTimerText")  -- Get cooldown timer text field by path.

   text = mainPanel:GetChildByPath("ButtonComp.ShootingImage.RemainingBulletsText")  -- Get remaining bullets text field by path.

   BulletCount = 10  -- Initialize bullet count to 10.
   controller1.selectedIndex = 0  -- Set controller state to shooting (0).
   controller2.selectedIndex = 0

-- Event handler for the Shooting Button; reduces bullet count when the button is clicked
   local function ButtonOnClickEvent()
       print("The button is clicked on.")
       if not isShooting then 
           return 
       end

       BulletCount = BulletCount - 1  -- Decrease bullet count by one.

       text.text = tostring(BulletCount)  -- Update the TextField to display the remaining bullet count

       if BulletCount <= 0 then
           controller1.selectedIndex = 1   -- Change controller state to disabled (1) when bullets run out.
           controller2.selectedIndex = 1
           isShooting = false  
       end
   end
   
   button.onClick:AddListener0(ButtonOnClickEvent)  -- Bind shooting button click event       
end

-- InputField and RichText functionality
function ChatInput()
   local chatHistoryData = {} -- Create a list to store chat messages

   local textInput = mainPanel:GetChildByPath("InputComp.Input") -- Get the field for user text input

   local sendButton = mainPanel:GetChildByPath("InputComp.SendButton") -- Get the send button for submitting messages

   local chatList = mainPanel:GetChildByPath("InputComp.ChatList") -- Get the chat list to display messages  

   -- Triggered when text is entered in the input field
   local function OnInputChangedEvent() 
       print(">>> Input value is : "..textInput.text) -- Print current input value when changed.
   end
   
   textInput.onChanged:AddListener0(OnInputChangedEvent) -- Listen for input change events.

   -- Method to render each item in the chat list
   local function OnRenderChatList(index, component)
       component:GetChild("Text").text = "[color=#54FF13] ".. chatHistoryData[index + 1].." [/color]" -- Render chat messages with color using UBB syntax. 
   end -- Use UBB syntax to set the color of the text in RichText format
   
   chatList.itemRenderer = OnRenderChatList -- Bind rendering method for chat list items.               

   -- Send message and update the chat list UI 
   local function SendButtonOnClickEvent() 
       if textInput.text == "" then return end 
       
       table.insert(chatHistoryData, textInput.text) -- Add input text to chat history data

       chatList.numItems = #chatHistoryData -- Update number of items in chat list based on history data length

       textInput.text = "" -- Clear input field after sending message. 
   end
   
   sendButton.onClick:AddListener0(SendButtonOnClickEvent) -- When Send button is clicked, send message and update UI
   
   textInput.onSubmit:AddListener0(SendButtonOnClickEvent) -- When Enter key is pressed, send message and update UI as well 
end

-- DropDown and Loader functionality
function DropDownAndLoader()
   local dropdown = mainPanel:GetChild("OccupationDropdown") -- Get the dropdown menu for selecting occupations

   local occupationText = mainPanel:GetChildByPath("OccupationLoader.OccupationText") -- Get the TextField to display selected occupation

   local loader = mainPanel:GetChildByPath("OccupationLoader.OccupationLoader") -- Get the loader to display corresponding images based on selection

-- Store image URLs corresponding to dropdown selections; change loader image based on selection change 
local loaderIcons =
{
     [1] = "ui://k94s5uzlj6vq5",
     [2] = "ui://k94s5uzlj6vq7",
     [3] = "ui://k94s5uzlj6vql",
     [4] = "ui://k94s5uzlj6vqm"
}

-- Change loader image based on dropdown selection change event. 
local function OnDropdownSelectedEvent() 
     occupationText.text = dropdown.text -- Update occupation text display based on selection 
     loader.url = loaderIcons[dropdown.selectedIndex + 1] -- Update loader image based on selected index 
end 

dropdown.onChanged:AddListener0(OnDropdownSelectedEvent) -- Bind change dropdown value button click event 
end

-- Graph: Change graph color functionality 
function ChangeGraphColor() 
     local graph = mainPanel:GetChildByPath("GraphComp.Graph") -- Get the graph component

     local changeColorButton=mainPanel:GetChildByPath("GraphComp.ChangeGraphButton")--Get the button for changing color 

     local function ChangeColorButtonOnClickEvent() 
      -- Randomize the graph color
         local randomValue1=math.random()  
         local randomValue2=math.random()  
         local randomValue3=math.random()  
         graph.color=UnityEngine.Color(randomValue1,randomValue2,randomValue3,0.5)--Set random color with transparency of graph. 
     end 

     changeColorButton.onClick:AddListener0(ChangeColorButtonOnClickEvent)--Bind change graph color button click event 
end

-- Create Sub Panel: Show Toast on button click functionality 
function AddGoldOrGem() 
     local function PlayTran(toastPanel) 
         toastPanel:GetTransition("Enter"):Play(function ()--Play enter transition effect. 
             toastPanel:GetTransition("Exit"):Play(function ()--Play exit transition effect after enter effect ends. 
                 toastPanel:Dispose()--Dispose of panel after exit transition ends. 
             end) 
         end) 
     end 

     local function CreateToastPanel() -- Method to Create Toast panel method
         local toastPanel=UIPackage.CreateObject(packageName,"Toast")
         mainPanel:AddChild(toastPanel)--Add Toast panel under main panel for display on screen. 
         toastPanel.size=mainPanel.size--Set Toast panel size to match screen size.  
         return toastPanel  
     end 

     local goldCount=0 -- Initialize gold count 
     local gemCount=0  -- Initialize gem count 

     local goldText=mainPanel:GetChildByPath("GoldAndGemComp.CoinText")  
     goldText.text=tostring(goldCount) -- Display initial gold count 

     local gemText=mainPanel:GetChildByPath("GoldAndGemComp.GemText")  
     gemText.text=tostring(gemCount) -- Display initial gem count 

     local addGoldButton=mainPanel:GetChildByPath("GoldAndGemComp.AddCoinButton")--Get button for adding gold 

     -- Set event handler for adding gold; update remaining gold count and create Toast panel 
     local function AddGoldButtonOnClickEvent()  
         goldCount=goldCount+1  
         goldText.text=tostring(goldCount)--Update remaining gold count display.  

         local toastPanel=CreateToastPanel()--Create Toast panel instance.  
         toastPanel:GetChild("ToastText").text="Gold Count + 1"--Set Toast message content.  
         PlayTran(toastPanel)--Play Toast display effect.  
     end 

     addGoldButton.onClick:AddListener0(AddGoldButtonOnClickEvent)--Bind add gold button click event

     local addGemButton=mainPanel:GetChildByPath("GoldAndGemComp.AddGemButton")--Get add gem button by path. 

      --Set add gem button click event to update gem count and create Toast panel.  
      local function AddGemButtonOnClickEvent()  
          gemCount=gemCount+1  
          gemText.text=tostring(gemCount)--Update remaining gem count display.  

          local toastPanel=CreateToastPanel()--Create Toast panel instance.  
          toastPanel:GetChild("ToastText").text="Gem Count + 1"--Set Toast message content.  
          PlayTran(toastPanel)--Play Toast display effect.  
      end 

      addGemButton.onClick:AddListener0(AddGemButtonOnClickEvent)--Bind add gem button click event.  
end 

-- Textfield and Progress functionality 
local time=0   
local totalTime=120--Set total duration for countdown timer 

function CountDownText()   
      if not mainPanel then return end 

      time=time+0.02--Increment time based on frame duration of .02 seconds 

      local countDownText=mainPanel:GetChildByPath("ProgressComp.Countdown")--Get countdown timer TextField by path 

      countDownText.text="Remaining Duration : "..math.ceil(totalTime-time).." S"--Update countdown timer display with remaining duration 

      local progress=mainPanel:GetChildByPath("ProgressComp.CountdownProgress")--Get Progress bar by path 

      progress.value=((totalTime-time)/totalTime)*100--Calculate remaining time ratio based on total duration; update progress bar value   
end 

local shootingTime=0   
local countDownTime=5--Set bullet cooldown time 

function ShootingCountDown()   
      if not isShooting then   
          shootingTime=shootingTime+0.02   

          if shootingTime>=countDownTime then--When cooldown countdown ends, set controller state back to shooting (0).   
              isShooting=true   
              controller1.selectedIndex=0   
              controller2.selectedIndex=0   
              BulletCount=10   
              text.text=tostring(BulletCount)--Update displayed bullet count back to total amount   
          else   
              shootingCountDown.text=math.ceil(countDownTime-shootingTime).." S"--Update cooldown countdown timer display   
          end   
      end   
end   

script.OnDispose(function ()   
      --Dispose of panel when script is disposed of    
      mainPanel:Dispose()    
      --Unload UIField resource    
      YaResourceManager.RemoveResourceByUIPackageField(UIField)    
end)